package net.starmiddleware.starws2.encoding;

import javax.xml.namespace.*;
import javax.xml.rpc.*;

import org.apache.axis.components.logger.LogFactory;
import org.apache.axis.encoding.*;
import org.apache.axis.utils.*;
import org.apache.commons.logging.*;

/**
 * @author Rich Scheuerle (scheu@us.ibm.com)
 *
 * This is the implementation of the axis TypeMapping interface (which extends
 * the JAX-RPC TypeMapping interface).
 *
 * A TypeMapping is obtained from the singleton TypeMappingRegistry using
 * the namespace of the webservice.  The TypeMapping contains the tuples
 * {Java type, SerializerFactory, DeserializerFactory, Type QName)
 *
 * So if you have a Web Service with the namespace "XYZ", you call
 * the TypeMappingRegistry.getTypeMapping("XYZ").
 *
 * The wsdl in your web service will use a number of types.  The tuple
 * information for each of these will be accessed via the TypeMapping.
 *
 * Because every web service uses the soap, schema, wsdl primitives, we could
 * pre-populate the TypeMapping with these standard tuples.  Instead,
 * if the namespace/class matches is not found in the TypeMapping
 * the request is delegated to the
 * Default TypeMapping or another TypeMapping
 *
 */
public class CORBATypeMappingImpl
    implements TypeMapping {
  protected static Log log =
      LogFactory.getLog(CORBATypeMappingImpl.class.getName());

  private Class complexType = null;
  private QName xmlType = new QName("http://www.omg.org/IDL-Mapped/", "corba");
  private CORBAComplexDeserFactory deserFactory = new CORBAComplexDeserFactory();
  private CORBAComplexSerFactory serFactory = new CORBAComplexSerFactory();

  /**
   * Should we "auto-type" classes we don't recognize into the "java:"
   * namespace?
   */
  private boolean doAutoTypes = false;

  /**
   * Construct TypeMapping
   */
  public CORBATypeMappingImpl() {
    complexType = net.starmiddleware.starws2.encoding.CORBA.ComplexType.class;
  }

  /**
   * setDelegate sets the new Delegate TypeMapping
   */
  public void setDelegate(TypeMapping delegate) {

  }

  /**
   * getDelegate gets the new Delegate TypeMapping
   */
  public TypeMapping getDelegate() {
    return null;
  }

  /********* JAX-RPC Compliant Method Definitions *****************/

  /**
   * Gets the list of encoding styles supported by this TypeMapping object.
   *
   * @return  String[] of namespace URIs for the supported encoding
   * styles and XML schema namespaces.
   */
  public String[] getSupportedEncodings() {

    return null;
  }

  /**
   * Sets the list of encoding styles supported by this TypeMapping object.
   * (Not sure why this is useful...this information is automatically updated
   * during registration.
   *
   * @param namespaceURIs String[] of namespace URI's
   */
  public void setSupportedEncodings(String[] namespaceURIs) {

  }

  /**
   * isRegistered returns true if the [javaType, xmlType]
   * pair is registered.
   * @param javaType - Class of the Java type
   * @param xmlType - Qualified name of the XML data type
   * @return true if there is a mapping for the given pair, or
   * false if the pair is not specifically registered.
   *
   * For example if called with (java.lang.String[], soapenc:Array)
   * this routine will return false because this pair is
   * probably not specifically registered.
   * However if getSerializer is called with the same pair,
   * the default TypeMapping will use extra logic to find
   * a serializer (i.e. array serializer)
   */
  public boolean isRegistered(Class javaType, QName xmlType) {
    if ( (javaType == this.complexType) &&
        (xmlType.getNamespaceURI().equals(this.xmlType.getNamespaceURI())))
      return true;
    return false;
  }

  /**
   * Registers SerializerFactory and DeserializerFactory for a
   * specific type mapping between an XML type and Java type.
   *
   * @param javaType - Class of the Java type
   * @param xmlType - Qualified name of the XML data type
   * @param sf - SerializerFactory
   * @param dsf - DeserializerFactory
   *
   * @throws JAXRPCException - If any error during the registration
   */
  public void register(Class javaType, QName xmlType,
                       javax.xml.rpc.encoding.SerializerFactory sf,
                       javax.xml.rpc.encoding.DeserializerFactory dsf) throws
      JAXRPCException {
    // At least a serializer or deserializer factory must be specified.
    if (sf == null && dsf == null) {
      throw new JAXRPCException(Messages.getMessage("badSerFac"));
    }

    internalRegister(javaType, xmlType, sf, dsf);
  }

  /**
   * Internal version of register(), which allows null factories.
   *
   * @param javaType
   * @param xmlType
   * @param sf
   * @param dsf
   * @throws JAXRPCException
   */
  protected void internalRegister(Class javaType, QName xmlType,
                                  javax.xml.rpc.encoding.SerializerFactory sf,
                                  javax.xml.rpc.encoding.DeserializerFactory
                                  dsf) throws JAXRPCException {

  }

  /**
   * Gets the SerializerFactory registered for the specified pair
   * of Java type and XML data type.
   *
   * @param javaType - Class of the Java type
   * @param xmlType - Qualified name of the XML data type
   *
   * @return Registered SerializerFactory
   *
   * @throws JAXRPCException - If there is no registered SerializerFactory
   * for this pair of Java type and XML data type
   * java.lang.IllegalArgumentException -
   * If invalid or unsupported XML/Java type is specified
   */
  public javax.xml.rpc.encoding.SerializerFactory
      getSerializer(Class javaType, QName xmlType) throws JAXRPCException {

    if (xmlType.getNamespaceURI().equals(this.xmlType.getNamespaceURI())) {
      return this.serFactory;
    }
    if (javaType == this.complexType)
      return this.serFactory;
    return null;

  }

  /**
   * Get the exact XML type QName which will be used when serializing a
   * given Class to a given type QName.  In other words, if we have:
   *
   * Class        TypeQName
   * ----------------------
   * Base         myNS:Base
   * Child        myNS:Child
   *
   * and call getXMLType(Child.class, BASE_QNAME), we should get
   * CHILD_QNAME.
   *
   * @param javaType
   * @param xmlType
   * @return the type's QName
   * @throws JAXRPCException
   */
  public QName getXMLType(Class javaType, QName xmlType) throws JAXRPCException {
    javax.xml.rpc.encoding.SerializerFactory sf = null;

    // If the xmlType was not provided, get one
    if (xmlType == null) {
      return this.xmlType;
    }

    if (javaType == this.complexType)
      return xmlType;

    if (xmlType.getNamespaceURI().equals(this.xmlType.getNamespaceURI()))
      return xmlType;
    return null;
  }

  /**
   * Gets the DeserializerFactory registered for the specified pair
   * of Java type and XML data type.
   *
   * @param javaType - Class of the Java type
   * @param xmlType - Qualified name of the XML data type
   *
   * @return Registered DeserializerFactory
   *
   * @throws JAXRPCException - If there is no registered DeserializerFactory
   * for this pair of Java type and  XML data type
   * java.lang.IllegalArgumentException -
   * If invalid or unsupported XML/Java type is specified
   */
  public javax.xml.rpc.encoding.DeserializerFactory
      getDeserializer(Class javaType, QName xmlType) throws JAXRPCException {
    javax.xml.rpc.encoding.DeserializerFactory df = null;

    if (xmlType.getNamespaceURI().equals(this.xmlType.getNamespaceURI())) {
      return this.deserFactory;
    }
    if (javaType == this.complexType)
      return this.deserFactory;
    return null;
  }

  /**
   * Removes the SerializerFactory registered for the specified
   * pair of Java type and XML data type.
   *
   * @param javaType - Class of the Java type
   * @param xmlType - Qualified name of the XML data type
   *
   * @throws JAXRPCException - If there is error in
   * removing the registered SerializerFactory
   */
  public void removeSerializer(Class javaType, QName xmlType) throws
      JAXRPCException {

  }

  /**
   * Removes the DeserializerFactory registered for the specified
   * pair of Java type and XML data type.
   *
   * @param javaType - Class of the Java type
   * @param xmlType - Qualified name of the XML data type
   *
   * @throws JAXRPCException - If there is error in
   * removing the registered DeserializerFactory
   */
  public void removeDeserializer(Class javaType, QName xmlType) throws
      JAXRPCException {

  }

  /********* End JAX-RPC Compliant Method Definitions *****************/

  /**
   * Gets the QName for the type mapped to Class.
   * @param javaType class or type
   * @return xmlType qname or null
   */
  public QName getTypeQNameRecursive(Class javaType) {
    QName ret = null;
    while (javaType != null) {
      ret = getTypeQName(javaType);
      if (ret != null)
        return ret;

      // Walk my interfaces...
      Class[] interfaces = javaType.getInterfaces();
      if (interfaces != null) {
        for (int i = 0; i < interfaces.length; i++) {
          Class iface = interfaces[i];
          ret = getTypeQName(iface);
          if (ret != null)
            return ret;
        }
      }

      javaType = javaType.getSuperclass();
    }
    return null;
  }

  public QName getTypeQName(Class javaType) {
    if (javaType == this.complexType)
      return xmlType;
    return null;
  }

  /**
   * Gets the Class mapped to QName.
   * @param xmlType qname or null
   * @return javaType class or type
   */
  public Class getClassForQName(QName xmlType) {
    if (xmlType == null)
      return null;

    //start-shuichao:for corba
    if (xmlType.getNamespaceURI().equals(this.xmlType.getNamespaceURI())) {
      return this.complexType;
    }

    return null;
  }

  /**
   * Gets the SerializerFactory registered for the Java type.
   *
   * @param javaType - Class of the Java type
   *
   * @return Registered SerializerFactory
   *
   * @throws JAXRPCException - If there is no registered SerializerFactory
   * for this pair of Java type and XML data type
   * java.lang.IllegalArgumentException -
   * If invalid or unsupported XML/Java type is specified
   */
  public javax.xml.rpc.encoding.SerializerFactory
      getSerializer(Class javaType) throws JAXRPCException {
    return getSerializer(javaType, null);
  }

  /**
   * Gets the DeserializerFactory registered for the xmlType.
   *
   * @param xmlType - Qualified name of the XML data type
   *
   * @return Registered DeserializerFactory
   *
   * @throws JAXRPCException - If there is no registered DeserializerFactory
   * for this pair of Java type and  XML data type
   * java.lang.IllegalArgumentException -
   * If invalid or unsupported XML/Java type is specified
   */
  public javax.xml.rpc.encoding.DeserializerFactory
      getDeserializer(QName xmlType) throws JAXRPCException {
    return getDeserializer(null, xmlType);
  }

  public void setDoAutoTypes(boolean doAutoTypes) {
    this.doAutoTypes = doAutoTypes;
  }
}
