package net.starmiddleware.starws2.security.rbac;

import java.net.URI;
import java.net.URISyntaxException;

import java.util.ArrayList;
import java.util.List;

import com.sun.xacml.*;
import com.sun.xacml.attr.*;
import com.sun.xacml.combine.*;
import com.sun.xacml.cond.*;
import com.sun.xacml.ctx.*;
import com.sun.xacml.finder.*;
import com.sun.xacml.finder.impl.*;

/**
 * <p>Title: </p>
 * <p>
 * This is an example program that shows how to build and generate an XACML
 * Policy. This doesn't show all the available features, but it provides a
 * good sample of commonly used functionality. Note that there's a fair amount
 * of duplicate code in this class, since verbosity (in this case) makes the
 * code a little easier to understand. An equivalent Polict to that generated
 * here is found in the policy directory as generated.xml. The generated
 * request can be used with this policy.
 * </p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: </p>
 * @author zm
 * @version 1.0
 */

public class PolicyProducer {

    /**
     * Simple helper routine that creates a TargetMatch instance.
     *
     * @param type the type of match
     * @param functionId the matching function identifier
     * @param designator the AttributeDesignator used in this match
     * @param value the AttributeValue used in this match
     *
     * @return the matching element
     */

    public static TargetMatch createTargetMatch(int type, String functionId,
						AttributeDesignator designator,
						AttributeValue value) {
	try {
	    // get the factory that handles Target functions and get an
	    // instance of the right function
	    FunctionFactory factory = FunctionFactory.getTargetInstance();
	    Function function = factory.createFunction(functionId);

	    // create the TargetMatch
	    return new TargetMatch(type, function, designator, value);
	}
	catch (Exception e) {
	    // note that in this example, we should never hit this case, but
	    // in the real world you need to worry about exceptions, especially
	    // from the factory
	    return null;
	}
    }

    /**
     * Creates the Target used in the Policy. This Target specifies that
     * the Policy applies to any example.com users who are requesting some
     * form of access to server.example.com.
     *
     * @return the target
     *
     * @throws URISyntaxException if there is a problem with any of the URIs
     */

    public static Target createPolicyTarget(String role) throws
	URISyntaxException {
	List subjects = new ArrayList();

	// create the Subject section
	List subject = new ArrayList();

	String subjectMatchId = EqualFunction.NAME_STRING_EQUAL;

	URI subjectDesignatorType = new URI(StringAttribute.identifier);

	URI subjectDesignatorId = new URI("role");

	AttributeDesignator subjectDesignator =
	    new AttributeDesignator(AttributeDesignator.SUBJECT_TARGET,
				    subjectDesignatorType, subjectDesignatorId, false);

	StringAttribute subjectValue = new StringAttribute(role);

	subject.add(createTargetMatch(TargetMatch.SUBJECT, subjectMatchId,
				      subjectDesignator, subjectValue));

	// put the Subject sections into their lists
	subjects.add(subject);

	// create & return the new Target
	return new Target(subjects, null, null);
    }

    /**
     * Creates the Target used in the Condition. This Target specifies that
     * the Condition applies to anyone taking the action commit.
     *
     * @return the target
     *
     * @throws URISyntaxException if there is a problem with any of the URIs
     */

    public static Target createRuleTarget(String service, String method) throws
	URISyntaxException {
	List resources = new ArrayList();

	// create the Resource section
	List resource = new ArrayList();

	String resourceMatchId = EqualFunction.NAME_STRING_EQUAL;

	URI resourceDesignatorType = new URI(StringAttribute.identifier);

	URI resourceDesignatorId = new URI(EvaluationCtx.RESOURCE_ID);

	AttributeDesignator resourceDesignator =
	    new AttributeDesignator(AttributeDesignator.RESOURCE_TARGET,
				    resourceDesignatorType,
				    resourceDesignatorId, false);

	StringAttribute resourceValue = new StringAttribute(service);

	resource.add(createTargetMatch(TargetMatch.RESOURCE, resourceMatchId,
				       resourceDesignator, resourceValue));

	// put the Resource sections into their lists
	resources.add(resource);

	List actions = new ArrayList();

	// create the Action section
	List action = new ArrayList();

	String actionMatchId = EqualFunction.NAME_STRING_EQUAL;

	URI actionDesignatorType = new URI(StringAttribute.identifier);

	URI actionDesignatorId = new URI("method");

	AttributeDesignator actionDesignator =
	    new AttributeDesignator(AttributeDesignator.ACTION_TARGET,
				    actionDesignatorType,
				    actionDesignatorId, false);

	StringAttribute actionValue = new StringAttribute(method);

	action.add(createTargetMatch(TargetMatch.ACTION, actionMatchId,
				     actionDesignator, actionValue));

	// put the Action section in the Actions list
	actions.add(action);

	// create & return the new Target
	return new Target(null, resources, actions);
    }

    static public List createAction(String method) throws URISyntaxException {
	// create the Action section
	List action = new ArrayList();

	String actionMatchId = EqualFunction.NAME_STRING_EQUAL;

	URI actionDesignatorType = new URI(StringAttribute.identifier);

	URI actionDesignatorId = new URI("method");

	AttributeDesignator actionDesignator =
	    new AttributeDesignator(AttributeDesignator.ACTION_TARGET,
				    actionDesignatorType,
				    actionDesignatorId, false);

	StringAttribute actionValue = new StringAttribute(method);

	action.add(createTargetMatch(TargetMatch.ACTION, actionMatchId,
				     actionDesignator, actionValue));
	return action;

    }

    /**
     * Creates the Rule used in the Policy.
     *
     * @return the rule
     *
     * @throws URISyntaxException if there is a problem with any of the URIs
     */

    public static Rule createRule(String service, String method) throws
	URISyntaxException {
	// define the identifier for the rule
	URI ruleId = new URI(service);

	// define the effect for the Rule
	int effect = Result.DECISION_PERMIT;

	// get the Target for the rule
	Target target = createRuleTarget(service, method);

	return new Rule(ruleId, effect, null, target, null);
    }

    /**
     * Command-line routine that bundles together all the information needed
     * to create a Policy and then encodes the Policy, printing to standard
     * out.
     */
    public static void main(String[] args) throws Exception {

	//role name
	String role = "employee";

	String service = "urn:xmltoday-delayed-quotes";
	String method = "getQuote";

	makePolicyFile(role, service, method);
    }


    public static void makePolicyFile(String role, String service,
				      String method) throws Exception {

	String id = role + "Policy";

	URI policyId = new URI(id);

	// get the combining algorithm for the policy
	URI combiningAlgId = new URI(PermitOverridesRuleAlg.algId);
	RuleCombiningAlgorithm combiningAlg =
	    (RuleCombiningAlgorithm) (CombiningAlgFactory.
				      createCombiningAlg(combiningAlgId));

	// create the target for the policy
	Target policyTarget = createPolicyTarget(role);
	Rule rule = createRule(service, method);

	// create a list for the rules and add our rules in order
	List ruleList = new ArrayList();
	ruleList.add(rule);

	// create the policy
	Policy policy = new Policy(policyId, combiningAlg, null,
				   policyTarget, ruleList);

	// finally, encode the policy and print it to standard out
	policy.encode(System.out, new Indenter());

	//test!!
	java.io.File file = new java.io.File( PermissionManager.POLICY_REPOSITORY + role + ".xml");
	java.io.FileOutputStream output = new java.io.FileOutputStream(file);
	//java.io.PrintStream out = new java.io.PrintStream(output);
	//out.println("<?xml version="1.0" encoding="UTF-8"?>");
	policy.encode(output);
	output.close();

    }
}
